#line 2 "qp-lex.c"

#line 4 "qp-lex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer qp_yy_create_buffer
#define yy_delete_buffer qp_yy_delete_buffer
#define yy_flex_debug qp_yy_flex_debug
#define yy_init_buffer qp_yy_init_buffer
#define yy_flush_buffer qp_yy_flush_buffer
#define yy_load_buffer_state qp_yy_load_buffer_state
#define yy_switch_to_buffer qp_yy_switch_to_buffer
#define yyin qp_yyin
#define yyleng qp_yyleng
#define yylex qp_yylex
#define yylineno qp_yylineno
#define yyout qp_yyout
#define yyrestart qp_yyrestart
#define yytext qp_yytext
#define yywrap qp_yywrap
#define yyalloc qp_yyalloc
#define yyrealloc qp_yyrealloc
#define yyfree qp_yyfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE qp_yyrestart(qp_yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int qp_yyleng;

extern FILE *qp_yyin, *qp_yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up qp_yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up qp_yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via qp_yyrestart()), so that the user can continue scanning by
	 * just pointing qp_yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when qp_yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int qp_yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow qp_yywrap()'s to do buffer switches
 * instead of setting up a fresh qp_yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void qp_yyrestart (FILE *input_file  );
void qp_yy_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE qp_yy_create_buffer (FILE *file,int size  );
void qp_yy_delete_buffer (YY_BUFFER_STATE b  );
void qp_yy_flush_buffer (YY_BUFFER_STATE b  );
void qp_yypush_buffer_state (YY_BUFFER_STATE new_buffer  );
void qp_yypop_buffer_state (void );

static void qp_yyensure_buffer_stack (void );
static void qp_yy_load_buffer_state (void );
static void qp_yy_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER qp_yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE qp_yy_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE qp_yy_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE qp_yy_scan_bytes (yyconst char *bytes,int len  );

void *qp_yyalloc (yy_size_t  );
void *qp_yyrealloc (void *,yy_size_t  );
void qp_yyfree (void *  );

#define yy_new_buffer qp_yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        qp_yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            qp_yy_create_buffer(qp_yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        qp_yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            qp_yy_create_buffer(qp_yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *qp_yyin = (FILE *) 0, *qp_yyout = (FILE *) 0;

typedef int yy_state_type;

extern int qp_yylineno;

int qp_yylineno = 1;

extern char *qp_yytext;
#define yytext_ptr qp_yytext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up qp_yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	qp_yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 31
#define YY_END_OF_BUFFER 32
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[149] =
    {   0,
       29,   29,   29,   29,   27,   27,   29,   29,   32,   30,
       29,   28,   30,   24,   30,   30,   30,   24,   20,   19,
       24,   20,   12,   12,   24,   18,   18,   18,   14,    1,
       24,   26,   27,   27,    8,    8,    7,    8,    8,    8,
        2,    3,   29,   28,    0,    0,   15,    0,    0,   25,
        0,    0,    4,   22,   13,    9,   13,   12,    0,    0,
       21,   18,   18,   14,    0,    0,    0,   26,   27,   27,
        7,    0,    5,    6,    0,   25,    0,    0,   23,    9,
       13,    0,   13,   12,    0,    0,   26,   25,    0,    0,
        0,    0,    0,    0,   26,   25,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   10,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   16,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,   11,    0,    0,    0,
        0,   17,    0,    0,    0,    0,    0,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    5,    6,    7,    8,    5,    5,    9,   10,
       11,   12,   13,    5,   13,   14,   15,   16,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   18,    5,   19,
       20,   21,    5,    1,   22,   22,   22,   22,   23,   22,
       24,   24,   24,   24,   24,   24,   24,   24,   24,   24,
       24,   24,   24,   24,   24,   24,   24,   25,   24,   24,
        5,   26,    5,    5,   24,    1,   22,   22,   22,   22,

       27,   22,   24,   24,   28,   24,   24,   29,   24,   30,
       24,   24,   24,   24,   24,   24,   24,   24,   24,   25,
       24,   24,   31,    5,   32,    5,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[33] =
    {   0,
        1,    2,    3,    4,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    5,    1,    6,    6,    1,    1,    1,
        1,    7,    7,    8,    8,    1,    7,    8,    8,    8,
        1,    1
    } ;

static yyconst flex_int16_t yy_base[168] =
    {   0,
        0,  281,   31,   60,   34,   37,   89,  120,  287,  435,
      282,  435,   39,  435,   39,  280,   44,  269,  435,  435,
       33,  265,  139,  254,  257,   31,  435,  256,    0,  435,
       42,  261,    0,  272,  435,  271,  435,   63,  260,  258,
      435,  435,  266,  435,   65,   68,  435,  264,  263,  248,
       68,   59,  435,  247,   79,    0,   81,    0,   94,    0,
      435,  435,  435,    0,   98,  230,    0,  243,    0,  253,
      435,   74,  435,  435,    0,  218,   63,  165,  435,    0,
      102,   96,  100,    0,  124,  198,  172,  166,  191,  125,
      117,  126,  103,  138,  140,  117,  130,    0,  119,  143,

      143,  126,  152,  217,  144,  100,  157,  159,  243,  135,
      156,  161,   83,  435,  168,  173,    0,  178,  181,  194,
      185,  269,  185,  189,  202,  435,  193,  200,  206,   69,
      215,  208,    0,  210,  221,   45,  435,  219,  295,  225,
      241,  435,  230,  235,    0,  237,  247,  435,  322,  330,
      338,  346,  353,  356,  360,  368,  372,  380,  383,  384,
      386,  390,  394,  402,  410,  418,  426
    } ;

static yyconst flex_int16_t yy_def[168] =
    {   0,
      148,    1,  149,  149,  150,  150,  151,  151,  148,  148,
      148,  148,  148,  148,  152,  153,  148,  148,  148,  148,
      148,  148,  148,   23,  148,  148,  148,  148,  154,  148,
      148,  155,  156,  156,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  152,  148,  152,  153,  157,
      148,  148,  148,  148,  148,  158,  148,   24,  148,  159,
      148,  148,  148,  154,  148,  148,  160,  155,  156,  156,
      148,  148,  148,  148,  161,  157,  148,  148,  148,  158,
      148,  148,  148,  159,  148,  148,  162,  163,  148,  148,
       89,  148,  148,  148,  162,  163,  148,   91,  148,  148,

      164,  148,  148,  148,  164,  165,  164,  148,  148,  148,
      109,  148,  165,  148,  148,  148,  111,  148,  148,  148,
      148,  148,  166,  122,  148,  148,  124,  148,  166,  167,
      166,  148,  127,  148,  148,  167,  148,  148,  148,  139,
      148,  148,  140,  148,  143,  148,  148,    0,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148
    } ;

static yyconst flex_int16_t yy_nxt[468] =
    {   0,
       10,   11,   12,   13,   14,   15,   14,   16,   17,   18,
       14,   19,   20,   21,   22,   23,   24,   25,   26,   27,
       28,   29,   29,   29,   29,   10,   29,   29,   29,   29,
       30,   14,   11,   12,   13,   34,   12,   13,   34,   12,
       13,   44,   45,   65,   47,   51,   54,  137,   55,   55,
       62,   63,   32,   32,   32,   32,   52,   32,   32,   32,
       32,   11,   12,   13,   48,   71,   72,   44,   45,   51,
       66,  137,   77,   47,   78,   78,   71,   72,   89,   89,
       52,   32,   32,   32,   32,  114,   32,   32,   32,   32,
       36,   37,   38,   48,   55,   55,   81,   81,   39,   65,

       40,   59,  114,   59,   93,   59,   82,   59,  101,   83,
       83,   83,   83,   85,   85,   83,   83,   81,   81,   41,
       42,   36,   37,   38,   59,   93,   90,  108,   59,   39,
       75,   40,   98,   98,  100,  100,  110,   97,   99,   85,
       85,  100,  100,  103,   90,  104,  104,  116,  106,  106,
       41,   42,   57,   67,   58,   58,  107,  107,  100,  100,
      108,   59,  106,   60,  102,   59,   90,  109,  109,  120,
      107,  117,  117,  118,  115,  115,  119,  119,   91,   75,
       78,   78,  110,  115,  115,   67,  121,   92,  122,  122,
      130,   92,   90,  119,  119,  120,  119,  119,  131,  123,

      124,  124,  148,  125,  124,  124,   89,   89,  133,  133,
      126,  130,  134,   92,  132,  135,  135,   92,  110,  131,
      130,  138,  125,  139,  139,  135,  135,   94,  131,  126,
      111,   75,  104,  104,  140,  140,  135,  135,  148,  112,
      140,  140,  141,  112,  110,  145,  145,  146,  141,  142,
      147,  147,  147,  147,   70,  142,   67,   86,  109,  109,
       79,   75,  147,  147,   49,  112,  148,   43,   74,  112,
      125,   73,   43,   70,   67,   62,   61,  126,  148,   56,
       53,   49,  127,   43,  122,  122,  148,   31,  148,  148,
      148,  128,  148,  148,  148,  128,  141,  148,  148,  148,

      148,  148,  148,  142,  148,  148,  148,  148,  143,  148,
      139,  139,  148,  148,  148,  148,  148,  144,  148,  148,
      148,  144,   10,   10,   10,   10,   10,   10,   10,   10,
       33,   33,   33,   33,   33,   33,   33,   33,   35,   35,
       35,   35,   35,   35,   35,   35,   46,   46,   46,   46,
       46,   46,   46,   46,   50,  148,  148,  148,  148,   50,
       50,   64,   64,   64,   68,   68,   68,   68,   69,   69,
      148,  148,   69,   69,   69,   69,   76,   76,   76,   76,
       80,   80,  148,   80,   80,   80,   80,   80,   84,   84,
       87,   87,   88,   88,   95,   95,   95,   95,   96,   96,

       96,   96,  105,  105,  148,  105,  105,  105,  105,  105,
      113,  113,  113,  113,  113,  113,  113,  113,  129,  129,
      148,  129,  129,  129,  129,  129,  136,  136,  136,  136,
      136,  136,  136,  136,    9,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148
    } ;

static yyconst flex_int16_t yy_chk[468] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    3,    3,    3,    5,    5,    5,    6,    6,
        6,   13,   13,   31,   15,   17,   21,  136,   21,   21,
       26,   26,    3,    3,    3,    3,   17,    3,    3,    3,
        3,    4,    4,    4,   15,   38,   38,   45,   45,   51,
       31,  130,   52,   46,   52,   52,   72,   72,   77,   77,
       51,    4,    4,    4,    4,  113,    4,    4,    4,    4,
        7,    7,    7,   46,   55,   55,   57,   57,    7,   65,

        7,   55,  106,   57,   93,   55,   59,   57,   93,   59,
       59,   82,   82,   65,   65,   83,   83,   81,   81,    7,
        7,    8,    8,    8,   81,   85,   90,  102,   81,    8,
       96,    8,   91,   91,   99,   99,  110,   90,   92,   85,
       85,   92,   92,   97,  100,   97,   97,  110,  101,  105,
        8,    8,   23,   95,   23,   23,  101,  105,  100,  100,
      108,   23,  107,   23,   94,   23,   78,  103,  103,  115,
      107,  111,  111,  112,  108,  108,  112,  112,   78,   88,
       78,   78,  119,  115,  115,   87,  116,   78,  116,  116,
      123,   78,   89,  118,  118,  120,  119,  119,  123,  120,

      121,  121,  124,  125,  124,  124,   89,   89,  127,  127,
      125,  129,  128,   89,  125,  128,  128,   89,  104,  129,
      131,  132,  135,  132,  132,  134,  134,   86,  131,  135,
      104,   76,  104,  104,  138,  138,  135,  135,  140,  104,
      140,  140,  141,  104,  109,  143,  143,  144,  147,  141,
      144,  144,  146,  146,   70,  147,   68,   66,  109,  109,
       54,   50,  147,  147,   49,  109,   48,   43,   40,  109,
      122,   39,   36,   34,   32,   28,   25,  122,   24,   22,
       18,   16,  122,   11,  122,  122,    9,    2,    0,    0,
        0,  122,    0,    0,    0,  122,  139,    0,    0,    0,

        0,    0,    0,  139,    0,    0,    0,    0,  139,    0,
      139,  139,    0,    0,    0,    0,    0,  139,    0,    0,
        0,  139,  149,  149,  149,  149,  149,  149,  149,  149,
      150,  150,  150,  150,  150,  150,  150,  150,  151,  151,
      151,  151,  151,  151,  151,  151,  152,  152,  152,  152,
      152,  152,  152,  152,  153,    0,    0,    0,    0,  153,
      153,  154,  154,  154,  155,  155,  155,  155,  156,  156,
        0,    0,  156,  156,  156,  156,  157,  157,  157,  157,
      158,  158,    0,  158,  158,  158,  158,  158,  159,  159,
      160,  160,  161,  161,  162,  162,  162,  162,  163,  163,

      163,  163,  164,  164,    0,  164,  164,  164,  164,  164,
      165,  165,  165,  165,  165,  165,  165,  165,  166,  166,
        0,  166,  166,  166,  166,  166,  167,  167,  167,  167,
      167,  167,  167,  167,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148,  148,  148,  148,
      148,  148,  148,  148,  148,  148,  148
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int qp_yy_flex_debug;
int qp_yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *qp_yytext;
#line 1 "../../../../tools/qfcc/source/qp-lex.l"
#line 2 "../../../../tools/qfcc/source/qp-lex.l"
/*
	qp-lex.l

	lexer for QuakePascal

	Copyright (C) 2011 Bill Currie <bill@taniwha.org>

	Author: Bill Currie <bill@taniwha.org>
	Date: 2011/01/06

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA

*/
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#include <ctype.h>

#include "QF/hash.h"

#include "debug.h"
#include "diagnostic.h"
#include "expr.h"
#include "grab.h"
#include "qfcc.h"
#include "shared.h"
#include "strpool.h"
#include "symtab.h"
#include "type.h"

#include "qp-parse.h"

#ifndef YY_PROTO
# define YY_PROTO(x) x
#else
# define YY_FLEX_REALLOC_HACK
#endif

#define YY_NO_UNPUT
#define YY_DECL int qp_yylex YY_PROTO(( void ))
YY_DECL;


static int keyword_or_id (const char *token);
static int convert_relop (const char *relop);


#line 694 "qp-lex.c"

#define INITIAL 0
#define GRAB_FRAME 1
#define GRAB_OTHER 2
#define COMMENT 3

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int qp_yylex_destroy (void );

int qp_yyget_debug (void );

void qp_yyset_debug (int debug_flag  );

YY_EXTRA_TYPE qp_yyget_extra (void );

void qp_yyset_extra (YY_EXTRA_TYPE user_defined  );

FILE *qp_yyget_in (void );

void qp_yyset_in  (FILE * in_str  );

FILE *qp_yyget_out (void );

void qp_yyset_out  (FILE * out_str  );

int qp_yyget_leng (void );

char *qp_yyget_text (void );

int qp_yyget_lineno (void );

void qp_yyset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int qp_yywrap (void );
#else
extern int qp_yywrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( qp_yytext, qp_yyleng, 1, qp_yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( qp_yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( qp_yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, qp_yyin))==0 && ferror(qp_yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(qp_yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int qp_yylex (void);

#define YY_DECL int qp_yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after qp_yytext and qp_yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( qp_yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(qp_yytext[qp_yyleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 84 "../../../../tools/qfcc/source/qp-lex.l"

					grab_frame = GRAB_FRAME;
					grab_other = GRAB_OTHER;

#line 892 "qp-lex.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! qp_yyin )
			qp_yyin = stdin;

		if ( ! qp_yyout )
			qp_yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			qp_yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				qp_yy_create_buffer(qp_yyin,YY_BUF_SIZE );
		}

		qp_yy_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of qp_yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 149 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 435 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 88 "../../../../tools/qfcc/source/qp-lex.l"
{ BEGIN (COMMENT); }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 89 "../../../../tools/qfcc/source/qp-lex.l"
{ warning (0, "nested { in comment"); }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 90 "../../../../tools/qfcc/source/qp-lex.l"
{ BEGIN (INITIAL); }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 91 "../../../../tools/qfcc/source/qp-lex.l"
{ BEGIN (COMMENT); }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 92 "../../../../tools/qfcc/source/qp-lex.l"
{ warning (0, "nested (* in comment"); }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 93 "../../../../tools/qfcc/source/qp-lex.l"
{ BEGIN (INITIAL); }
	YY_BREAK
case 7:
/* rule 7 can match eol */
YY_RULE_SETUP
#line 94 "../../../../tools/qfcc/source/qp-lex.l"
{ pr.source_line++; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 95 "../../../../tools/qfcc/source/qp-lex.l"
/* nothing to do, with people like you */
	YY_BREAK
case YY_STATE_EOF(COMMENT):
#line 96 "../../../../tools/qfcc/source/qp-lex.l"
{ error (0, "EOF in comment"); return 0; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 97 "../../../../tools/qfcc/source/qp-lex.l"
/* nothing to do, with people like you */
	YY_BREAK
case 10:
*yy_cp = (yy_hold_char); /* undo effects of setting up qp_yytext */
(yy_c_buf_p) = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up qp_yytext again */
YY_RULE_SETUP
#line 99 "../../../../tools/qfcc/source/qp-lex.l"
{ line_info (qp_yytext + 1); }
	YY_BREAK
case 11:
*yy_cp = (yy_hold_char); /* undo effects of setting up qp_yytext */
(yy_c_buf_p) = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up qp_yytext again */
YY_RULE_SETUP
#line 100 "../../../../tools/qfcc/source/qp-lex.l"
{ line_info (qp_yytext + 1); }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 102 "../../../../tools/qfcc/source/qp-lex.l"
{
						int         i;

						if (qp_yytext[0] == '0' && tolower (qp_yytext[1] == 'b'))
							i = strtol (qp_yytext + 2, 0, 2);
						else
							i = strtol (qp_yytext, 0, 0);
						qp_yylval.expr = new_integer_expr (i);
						return VALUE;
					}
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 113 "../../../../tools/qfcc/source/qp-lex.l"
{
						float       f = strtof (qp_yytext, 0);
						qp_yylval.expr = new_float_expr (f);
						return VALUE;
					}
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 119 "../../../../tools/qfcc/source/qp-lex.l"
return keyword_or_id (qp_yytext);
	YY_BREAK
case 15:
/* rule 15 can match eol */
YY_RULE_SETUP
#line 121 "../../../../tools/qfcc/source/qp-lex.l"
{
						const char *s = make_string (qp_yytext, 0);
						qp_yylval.expr = new_string_expr (s);
						return VALUE;
					}
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 127 "../../../../tools/qfcc/source/qp-lex.l"
{
						vec3_t      v;
						sscanf (qp_yytext, "' %f %f %f '",
								&v[0], &v[1], &v[2]);
						qp_yylval.expr = new_vector_expr (v);
						return VALUE;
					}
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 135 "../../../../tools/qfcc/source/qp-lex.l"
{
						quat_t      q;
						sscanf (qp_yytext, "' %f %f %f %f'",
								&q[0], &q[1], &q[2], &q[3]);
						qp_yylval.expr = new_quaternion_expr (q);
						return VALUE;
					}
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 143 "../../../../tools/qfcc/source/qp-lex.l"
{
						qp_yylval.op = convert_relop (qp_yytext);
						return RELOP;
					}
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 147 "../../../../tools/qfcc/source/qp-lex.l"
{
						qp_yylval.op = qp_yytext[0];
						return ADDOP;
					}
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 151 "../../../../tools/qfcc/source/qp-lex.l"
{
						qp_yylval.op = qp_yytext[0];
						return MULOP;
					}
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 156 "../../../../tools/qfcc/source/qp-lex.l"
return ASSIGNOP;
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 158 "../../../../tools/qfcc/source/qp-lex.l"
return RANGE;
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 159 "../../../../tools/qfcc/source/qp-lex.l"
return ELLIPSIS;
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 161 "../../../../tools/qfcc/source/qp-lex.l"
return qp_yytext[0];
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 163 "../../../../tools/qfcc/source/qp-lex.l"
{
						int ret = do_grab (qp_yytext);
						if (ret >= 0) {
							qp_yylval.expr = new_integer_expr (ret);
							return VALUE;
						} else {
							BEGIN (-ret);
						}
					}
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 173 "../../../../tools/qfcc/source/qp-lex.l"
add_frame_macro (qp_yytext);
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 174 "../../../../tools/qfcc/source/qp-lex.l"
/* skip */
	YY_BREAK
case 28:
/* rule 28 can match eol */
YY_RULE_SETUP
#line 176 "../../../../tools/qfcc/source/qp-lex.l"
{
						pr.source_line++;
						BEGIN (INITIAL);
					}
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 181 "../../../../tools/qfcc/source/qp-lex.l"
/* skip whitespace */
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 183 "../../../../tools/qfcc/source/qp-lex.l"
error (0, "all your typo are belong to us");
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 185 "../../../../tools/qfcc/source/qp-lex.l"
ECHO;
	YY_BREAK
#line 1193 "qp-lex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(GRAB_FRAME):
case YY_STATE_EOF(GRAB_OTHER):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed qp_yyin at a new source and called
			 * qp_yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = qp_yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( qp_yywrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * qp_yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of qp_yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					qp_yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			qp_yyrestart(qp_yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) qp_yyrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 149 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 149 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 148);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up qp_yytext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					qp_yyrestart(qp_yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( qp_yywrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve qp_yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void qp_yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        qp_yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            qp_yy_create_buffer(qp_yyin,YY_BUF_SIZE );
	}

	qp_yy_init_buffer(YY_CURRENT_BUFFER,input_file );
	qp_yy_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void qp_yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		qp_yypop_buffer_state();
	 *		qp_yypush_buffer_state(new_buffer);
     */
	qp_yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	qp_yy_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (qp_yywrap()) processing, but the only time this flag
	 * is looked at is after qp_yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void qp_yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	qp_yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE qp_yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) qp_yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in qp_yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) qp_yyalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in qp_yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	qp_yy_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with qp_yy_create_buffer()
 * 
 */
    void qp_yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		qp_yyfree((void *) b->yy_ch_buf  );

	qp_yyfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a qp_yyrestart() or at EOF.
 */
    static void qp_yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	qp_yy_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then qp_yy_init_buffer was _probably_
     * called from qp_yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void qp_yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		qp_yy_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void qp_yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	qp_yyensure_buffer_stack();

	/* This block is copied from qp_yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from qp_yy_switch_to_buffer. */
	qp_yy_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void qp_yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	qp_yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		qp_yy_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void qp_yyensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)qp_yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in qp_yyensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)qp_yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in qp_yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE qp_yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) qp_yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in qp_yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	qp_yy_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to qp_yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       qp_yy_scan_bytes() instead.
 */
YY_BUFFER_STATE qp_yy_scan_string (yyconst char * yystr )
{
    
	return qp_yy_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to qp_yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE qp_yy_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) qp_yyalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in qp_yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = qp_yy_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in qp_yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up qp_yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		qp_yytext[qp_yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = qp_yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		qp_yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int qp_yyget_lineno  (void)
{
        
    return qp_yylineno;
}

/** Get the input stream.
 * 
 */
FILE *qp_yyget_in  (void)
{
        return qp_yyin;
}

/** Get the output stream.
 * 
 */
FILE *qp_yyget_out  (void)
{
        return qp_yyout;
}

/** Get the length of the current token.
 * 
 */
int qp_yyget_leng  (void)
{
        return qp_yyleng;
}

/** Get the current token.
 * 
 */

char *qp_yyget_text  (void)
{
        return qp_yytext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void qp_yyset_lineno (int  line_number )
{
    
    qp_yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see qp_yy_switch_to_buffer
 */
void qp_yyset_in (FILE *  in_str )
{
        qp_yyin = in_str ;
}

void qp_yyset_out (FILE *  out_str )
{
        qp_yyout = out_str ;
}

int qp_yyget_debug  (void)
{
        return qp_yy_flex_debug;
}

void qp_yyset_debug (int  bdebug )
{
        qp_yy_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from qp_yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    qp_yyin = stdin;
    qp_yyout = stdout;
#else
    qp_yyin = (FILE *) 0;
    qp_yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * qp_yylex_init()
     */
    return 0;
}

/* qp_yylex_destroy is for both reentrant and non-reentrant scanners. */
int qp_yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		qp_yy_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		qp_yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	qp_yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * qp_yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *qp_yyalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *qp_yyrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void qp_yyfree (void * ptr )
{
	free( (char *) ptr );	/* see qp_yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 185 "../../../../tools/qfcc/source/qp-lex.l"



int
qp_yywrap (void)
{
	return 1;
}

typedef struct {
	const char *name;
	int         value;
	type_t     *type;
} keyword_t;

static keyword_t keywords[] = {
	{"real",		TYPE,		&type_float},
	{"string",		TYPE,		&type_string},
	{"vector",		TYPE,		&type_vector},
	{"entity",		TYPE,		&type_entity},
	{"quaternion",	TYPE,		&type_quaternion},
	{"integer",		TYPE,		&type_integer},

	{"program",		PROGRAM,	0},
	{"var",			VAR,		0},
	{"array",		ARRAY,		0},
	{"of",			OF,			0},
	{"function",	FUNCTION,	0},
	{"procedure",	PROCEDURE,	0},
	{"begin",		PBEGIN,		0},
	{"end",			END,		0},
	{"if",			IF,			0},
	{"then",		THEN,		0},
	{"else",		ELSE,		0},
	{"while",		WHILE,		0},
	{"do",			DO,			0},
	{"or",			ADDOP,		0},
	{"div",			MULOP,		0},
	{"mod",			MULOP,		0},
	{"and",			MULOP,		0},
	{"not",			NOT,		0},

	{"return",		RETURN,		0},
};

static const char *
keyword_get_key (const void *kw, void *unused)
{
	return ((keyword_t *) kw)->name;
}

static int
keyword_or_id (const char *token)
{
	static hashtab_t *keyword_tab;
	keyword_t  *keyword;
	symbol_t   *sym;

	if (!keyword_tab) {
		size_t      i;
		keyword_tab = Hash_NewTable (1021, keyword_get_key, 0, 0);
		for (i = 0; i < sizeof (keywords) / sizeof (keywords[0]); i++)
			Hash_Add (keyword_tab, &keywords[i]);
	}
	keyword = Hash_Find (keyword_tab, token);
	if (keyword) {
		if (keyword->value == ADDOP || keyword->value == MULOP) {
			qp_yylval.op = token[0];
		} else {
			qp_yylval.type = keyword->type;
		}
		return keyword->value;
	}
	sym = symtab_lookup (current_symtab, token);
	if (!sym)
		sym = new_symbol (token);
	if (sym->sy_type == sy_type) {
		qp_yylval.type = sym->type;
		return TYPE_NAME;
	}
	qp_yylval.symbol = sym;
	return ID;
}

static int
convert_relop (const char *relop)
{
	switch (relop[0]) {
		case '=':
			return EQ;
		case '<':
			switch (relop[1]) {
				case 0:
					return LT;
				case '>':
					return NE;
				case '=':
					return LE;
			}
			break;
		case '>':
			switch (relop[1]) {
				case 0:
					return GT;
				case '=':
					return GE;
			}
			break;
	}
	internal_error (0, "bad relop %s", relop);
}

#ifdef YY_FLEX_REALLOC_HACK
static __attribute__ ((used)) void *(*const yy_flex_realloc_hack)(void *,yy_size_t) = yy_flex_realloc;
#else
static __attribute__ ((used)) void (*yyunput_hack)(int, char*) = yyunput;
static __attribute__ ((used)) int (*input_hack)(void) = input;
#endif


